package main

import (
	"os"
	"os/user"
	"path/filepath"

	"go.wit.com/log"

	"go.wit.com/gui"
	"go.wit.com/lib/gadgets"
	"go.wit.com/lib/gui/repolist"
	"go.wit.com/lib/gui/repostatus"
)

func doesExist(path string) bool {
	if _, err := os.Stat(path); err != nil {
		if os.IsNotExist(err) {
			return false
		}
	}
	return true
}

// only errors on bad errors
func quickCmd(fullpath string, cmd []string) bool {
	var err error
	var b bool
	var output string

	//	if me.autoWorkingPwd.String() != fullpath {
	//		me.autoWorkingPwd.SetValue(fullpath)
	//	}

	if me.autoDryRun.Checked() {
		log.Warn("RUN --dry-run", fullpath, cmd)
		return false
	} else {
		log.Warn("RUN:", fullpath, cmd)
	}

	err, b, output = repostatus.RunCmd(fullpath, cmd)
	if err != nil {
		log.Warn("cmd =", cmd)
		log.Warn("err =", err)
		log.Warn("b =", b)
		log.Warn("output =", string(output))
		return false
	} else if !b {
		log.Warn("b =", b)
		log.Warn("output =", string(output))
		return true
	}
	log.Warn("output = ", string(output))
	return true
}

func globalBuildOptions(vbox *gui.Node) {
	group1 := vbox.NewGroup("Global Build Options")
	grid := group1.NewGrid("buildOptions", 0, 0)

	// me.autoWorkingPwd = gadgets.NewOneLiner(grid, "working directory (pwd)")
	me.userHomePwd = gadgets.NewOneLiner(grid, "user home")
	grid.NextRow()
	me.goSrcPwd = gadgets.NewOneLiner(grid, "go src home")
	grid.NextRow()

	usr, _ := user.Current()
	homeDir, err := os.UserHomeDir()
	if err != nil {
		log.Warn("Error getting home directory:", err)
		homeDir = "/home/autotypist"
	}
	me.userHomePwd.SetText(homeDir)
	srcDir := filepath.Join(homeDir, "go/src")
	me.goSrcPwd.SetText(srcDir)

	// select the branch you want to test, build and develop against
	// this lets you select your user branch, but, when you are happy
	// you can merge everything into the devel branch and make sure it actually
	// works. Then, when that is good, merge and version everything in master
	me.setBranchB = grid.NewButton("set current branch to:", func() {
		targetName := me.newBranch.String()
		log.Warn("setting all branches to", targetName)
		for _, repo := range repolist.AllRepos() {
			repo.Status.CheckoutBranch(targetName)
			repo.Scan()
		}
	})
	me.newBranch = grid.NewCombobox()
	me.newBranch.AddText("master")
	me.newBranch.AddText("devel")
	me.newBranch.AddText(usr.Username)
	me.newBranch.SetText(usr.Username)

	// checking this will automatically make the branches off of devel
	me.autoCreateBranches = grid.NewCheckbox("create if missing").SetChecked(true)
	grid.NextRow()
}
